﻿
using System;
using System.Windows.Forms;
using System.Diagnostics;

// To compute the singular value decomposition of a matrix
// www.mosimath.com


namespace WindowsFormsApplication1
{

     public partial class MainWin : Form
    {
        const int n = 6;      // y order
        const int m = 6;      // x order
        public TextBox[,] tbm;
        public TextBox[] tbs;
        public TextBox[] tbx;
 
        Stopwatch timer = new Stopwatch();

        public double[,] a = new double[n, m];   // starting matrix
        public double[,] ori = new double[n, m]; // origing matrix (copy of A)
        public double[,] r = new double[n, n];   // rotation matrix for Jacoby
        public double[,] v = new double[m, n];
        public double[,] u = new double[n, n];
        public double[] d = new double[m];
        public int rank = 0;

        public void CalcRMatrix(double[,] a, int p, int q, int size)
        {
            int i, j;
            double phi;
            double t;
            phi = (a[q, q] - a[p, p]) / 2 / a[q, p];
            if (phi != 0)
            {
                if (phi > 0)
                    t = 1/(phi + Math.Sqrt(phi * phi + 1.0));
                else
                    t = 1/(phi - Math.Sqrt(phi * phi + 1.0));
            }
            else
                t = 1.0;
            for (i = 0; i < size; i++)
            {
                for (j = 0; j < size; j++)
                {
                    r[i, j] = 0.0;
                }
                r[i, i] = 1.0;
            }
            r[p, p] = 1 / Math.Sqrt(t * t + 1.0);
            r[p, q] = t * r[p, p];
            r[q, q] = r[p, p];
            r[q, p] = -r[p, q];
        }


        public void RunJacobi(int maxOrder)
        {
            int i, k, count = 0;
            double delta = 10;
            while ((delta > 1.0E-40) && (count < 50))
            {
                delta = 0;
                count++;
                for (i = 0; i < maxOrder; i++)
                {
                    for (k = i + 1; k < maxOrder; k++)
                    {
                        if (a[k, i] != 0.0)
                        {
                            CalcRMatrix(a, i, k, maxOrder);
                            a = Matrix.MultTranspMatrixLeft(a, r, i, k, maxOrder);
                            a = Matrix.MultMatrixRight(a, r, i, k, maxOrder);
                            v = Matrix.MultMatrixRight(v, r, i, k, maxOrder);
                        }
                    }
                    if(i< maxOrder-1)
                        delta = delta + Math.Abs(a[i+1, i]);
                }
            }    
        }

        void FillCols(int empties)
        {
            int i, j;
            int index;
            TGauss Gauss = new TGauss(n);
            for (index = 0; index < empties; index++)
            {
                for (i = 0; i < n; i++)
                {
                    for (j = 0; j < n; j++)
                    {
                        Gauss.a[j, i] = u[i, j];
                    }
                    Gauss.y[i] = 0;
                }

                Gauss.Eliminate(empties);
                Gauss.Solve(empties - index);
                for (i = 0; i < Gauss.order; i++)
                {
                    u[i, n - empties + index] = Gauss.x[i];
                }
            }
        }

        public MainWin()
        {
            int i;
         
            InitializeComponent();

            GMatrixA.ColumnCount = m;
            dGEigenVectors.ColumnCount = m;
            dGEigenVectors.RowCount = n;

            uVectors.ColumnCount = n;
            uVectors.RowCount = n;

            dValues.ColumnCount = m;
            dValues.RowCount = n;
            for (i = 0; i < GMatrixA.ColumnCount; i++)
            {
                GMatrixA.Columns[i].Width = 160;
                dGEigenVectors.Columns[i].Width = 160;
            }
          
            // Sample matrix 6 * 6 rank = 4
           /* GMatrixA.Rows.Add(new object[] { 22, 52, 1, 93, 70, 33 });       
            GMatrixA.Rows.Add(new object[] { 68, 3, 7, 53, 76, 76 });
            GMatrixA.Rows.Add(new object[] { 22, 52, 1, 93, 70, 33 });
            GMatrixA.Rows.Add(new object[] { 68, 5, 42, 9, 26, 99 });
            GMatrixA.Rows.Add(new object[] { 5, 83, 38, 85, 91, 63 });
            GMatrixA.Rows.Add(new object[] { 68, 3, 7, 53, 76, 76 });*/


             // Sample matrix 6 * 6 regular
             GMatrixA.Rows.Add(new object[] { 22, 52, 1, 93, 70, 33 });
             GMatrixA.Rows.Add(new object[] { 5, 83, 38, 85, 91, 63 }); 
             GMatrixA.Rows.Add(new object[] { 68, 3, 7, 53, 76, 76 });
             GMatrixA.Rows.Add(new object[] { 68, 5, 42, 9, 26, 99 });
             GMatrixA.Rows.Add(new object[] { 93, 53, 69, 65, 5, 37 });
             GMatrixA.Rows.Add(new object[] { 38, 67, 59, 42, 74, 25 });

            // Sample matrix 6 * 4 set n = 6 and  m = 4;
           /* GMatrixA.Rows.Add(new object[] { 22, 52, 1, 93 });
            GMatrixA.Rows.Add(new object[] { 5, 83, 38, 85 });
            GMatrixA.Rows.Add(new object[] { 68, 3, 7, 53 });
            GMatrixA.Rows.Add(new object[] { 68, 5, 42, 9 });
            GMatrixA.Rows.Add(new object[] { 93, 53, 69, 65 });
            GMatrixA.Rows.Add(new object[] { 38, 67, 59, 42 });*/

        }

        private void button1_MouseClick(object sender, MouseEventArgs e)
        {
            int i, j, k;
            int index;
            double tempVal;
            double[,] buf = new double[n, m];
            DataGridViewCell cell;

            for (i = 0; i < m; i++)
            {
                for (k = 0; k < n; k++)
                {
                    cell = GMatrixA[i, k];  // get cell to access
                    a[k, i] = Convert.ToDouble(cell.Value);
                    ori[k, i] = a[k, i];

                    cell = dValues[i, k];
                    cell.Value = "";
                }
            }
            for (i = 0; i < m; i++) // initialise V 
            {
                for (k = 0; k < m; k++)
                {
                    v[k, i] = 0.0;
                }
                v[i, i] = 1.0;
            }

            // calculate  A^T * A 
            for (i = 0; i < m; i++)
            {
                for (j = 0; j < m; j++)
                {
                    buf[j, i] = 0.0;
                    for (k = 0; k < n; k++)
                    {
                        buf[j, i] = buf[j, i] + a[k, j] * a[k, i];
                    }
                }
            }
            for (i = 0; i < m; i++)
            {
                for (j = 0; j < m; j++)
                {
                    a[j, i] = buf[j, i];
                }
            }

            // calculate the eigenvalues
            RunJacobi(m);

            // calculate d
            for (i = 0; i < m; i++)
            {
                if (a[i, i] > 0)
                    d[i] = Math.Sqrt(a[i, i]);
                else
                    d[i] = 0;
            }


            // order values
            for (i = 0; i < m - 1; i++)
            {
                index = i;
                for (j = i + 1; j < m; j++)
                {

                    if (d[j] > d[index])
                        index = j;
                }

                tempVal = d[i];
                d[i] = d[index];
                d[index] = tempVal;
                for (j = 0; j < m; j++)
                {
                    tempVal = v[j, i];
                    v[j, i] = v[j, index];
                    v[j, index] = tempVal;
                }
            }


            rank = 1;
            for (i = 0; i < m; i++)
            {
                if (Math.Abs(d[i]) > 1E-5)
                    rank = i + 1;
            }

            // calculate u
            for (i = 0; i < rank; i++)
            {
                for (j = 0; j < n; j++)
                {
                    u[j, i] = 0;
                    for (k = 0; k < m; k++)
                    {
                        u[j, i] = u[j, i] + ori[j, k] * v[k, i];
                    }
                    if (d[i] != 0)
                        u[j, i] = u[j, i] / d[i];
                }
            }

            if ((rank > 1) && (rank < n))
                FillCols(n - rank);  // fill empty columns of U

            for (i = 0; i < m; i++)  // display results
            {
                for (k = 0; k < n; k++)
                {
                    cell = GMatrixA[i, k];  // get cell to access
                    cell.Value = Convert.ToString(Math.Round(a[k, i], 4));

                    if (k < m)
                    {
                        cell = dGEigenVectors[i, k];
                        cell.Value = Convert.ToString(Math.Round(v[k, i], 4));
                    }

                }
                cell = dValues[i, i];
                cell.Value = Convert.ToString(Math.Round(d[i], 4));
            }

            for (i = 0; i < n; i++)
            {
                for (k = 0; k < n; k++)
                {
                    cell = uVectors[i, k];
                    cell.Value = Convert.ToString(Math.Round(u[k, i], 4));

                }
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            int i, k;
            DataGridViewCell cell;
          
            // Check A = U*L*V^T
            double[,] tempA = new double[n, m];
            for (i = 0; i < m; i++)
            {
                tempA[i, i] = d[i];
            }
           
            tempA = Matrix.MultNormal(u, tempA, n, n, m, n);
            tempA = Matrix.MultTrans(tempA, v, m, n, m, m);

            for (i = 0; i < m; i++)
            {
                for (k = 0; k < n; k++)
                {
                    cell = GMatrixA[i, k];  // get cell to access
                    cell.Value = Convert.ToString(Math.Round(tempA[k, i],4));
                }
            }

        }    
    }
}
